==> emodpy_covid-0.0.30-py3-none-any.whl: emodpy_covid/mid/dtk_in_process_trace_and_isolate_infecteds.py <==
#!/usr/bin/python

import json
import sqlite3
import random
from . import schema_to_class as s2c
from . import dtk_in_process_qtine_fams as isofam

prob_finding_contacts = 1.0
verbose = False
symptomatic_fraction = 0.2
isofam.young_sympto_fraction = symptomatic_fraction # does this work?
max_trace_time = 2

def app( timestep ):
    # Do SQL query against simulation.db (?) and look for any NewlySymptomatic events this timestep
    # If there are any, for each individual, do a query for any NewInfection events within last 5 days
    # where the infector id == the id of the newly symptomatic individual.
    # Create a new campaign file which distributes SimpleVaccine to those individuals.
    # This will require creation of new method to target interventions by individual id

    # CREATE TABLE SIM_EVENTS (SIM_TIME       INT    NOT NULL,EVENT          TEXT     NOT NULL,INDIVIDUAL     INT    NOT NULL,MISC           INT    NOT NULL);

    newly_symptos = []
    victims_to_treat = []
    with sqlite3.connect('simulation_events.db') as conn:
        cur = conn.cursor()
        # test query
        query = "SELECT COUNT(*) from SIM_EVENTS;".format( timestep ) 
        cur.execute( query )
        rows = cur.fetchone()
        if rows[0] == 0:
            if verbose:
                print( "SIM_EVENTS db is empty! Ok at beginning of simulation but something might be wrong." )
            return ""

        query = "SELECT INDIVIDUAL, EVENT from SIM_EVENTS where SIM_TIME={} and ( EVENT='NewlySymptomatic' or EVENT='NewSevereCase' );".format( timestep ) 
        cur.execute( query )
        rows = cur.fetchall()
        for row in rows:
            infector = row[0]
            event    = row[1]
            if verbose:
                print( f"Found newly symptomatic individual {infector}. Look for victims..." )
            if event == "NewlySymptomatic":
                if random.random() < symptomatic_fraction:
                    newly_symptos.append( infector )
            elif event == "NewSevereCase":
                newly_symptos.append( infector )

        if len(newly_symptos)==0:
            if verbose:
                print( "Found NO newly symptomatic individuals to contact trace." )

        for infector in newly_symptos:
            if verbose:
                print( "Looking for victims of {} at time {}".format( infector, timestep ) )
            cur2 = conn.cursor()
            #query2 = "SELECT INDIVIDUAL from SIM_EVENTS where SIM_TIME<={} and SIM_TIME>{} and EVENT=='NewInfection' and MISC=={};".format( timestep, timestep-10, infector )
            query2 = "SELECT INDIVIDUAL from SIM_EVENTS where EVENT=='NewInfection' and MISC=={};".format( infector )
            cur2.execute( query2 )
            rows2 = cur2.fetchall()
            for row in rows2:
                #print( str( row ) )
                infected = row[0]
                if verbose:
                    print( f"Found infected victim {infected} of {infector}." )
                if random.random() < prob_finding_contacts:
                    victims_to_treat.append( infected )

    if len( victims_to_treat ) > 0:
        if verbose:
            print( "Going to distribute vaccines to {}.".format( victims_to_treat ) )
    else:
        if verbose:
            print( "No infectees/victims found. Not distributing any 'contact tracing' isolations this time." )
        return ""

    # DelayedIV -> MultiIV -> RASV & BroadcasttEvent
    event = s2c.get_default_campaign_event() 
    coordinator = s2c.get_class_with_defaults( "StandardEventCoordinator" )
    if coordinator is None:
        print( "s2c.get_class_with_defaults returned None. Maybe no schema.json was provided." )
        return "" 
    intervention = s2c.get_class_with_defaults( "DelayedIntervention" ) 
    act_intervention = s2c.get_class_with_defaults( "MultiInterventionDistributor" ) 
    act_act_intervention = s2c.get_class_with_defaults( "RouteAwareSimpleVaccine" ) 
    waning = s2c.get_class_with_defaults( "WaningEffectBox" )
    act_act_co_intervention = s2c.get_class_with_defaults( "BroadcastEvent" ) 

    coordinator.Target_Demographic = "ExplicitIDs"
    coordinator.Max_Cases_Per_Node  = 10
    intervention.Delay_Period_Distribution = "UNIFORM_DISTRIBUTION"
    intervention.Delay_Period_Min = 0
    intervention.Delay_Period_Max = max_trace_time
    act_act_intervention.Vaccine_Type = "Generic"
    act_act_intervention.Vaccine_Route = "TRANSMISSIONROUTE_CONTACT" 
    waning.Initial_Effect = 0.9
    waning.Box_Duration = 14
    act_act_co_intervention.Broadcast_Event = "IsolateAtHome"

    coordinator.Intervention_Config = intervention 
    event.Event_Coordinator_Config = coordinator
    intervention.Actual_IndividualIntervention_Configs = [ act_intervention ]
    act_intervention.Intervention_List = [ act_act_intervention, act_act_co_intervention ]
    act_act_intervention.Waning_Config = waning 
    
    event.Start_Day = float(timestep+1)
    event.Event_Coordinator_Config.ID_List = victims_to_treat

    campaign = {}
    campaign["Events"] = []
    campaign["Events"].append( event )

    fam_events = isofam.get_events( timestep )
    for event in fam_events:
        campaign["Events"].append( event )

    with open( "contact_trace.json", "w" ) as camp_file:
        json.dump( campaign, camp_file )
    return "contact_trace.json"

==> emodpy_covid-0.0.32-py3-none-any.whl: emodpy_covid/mid/dtk_in_process_trace_and_isolate_infecteds.py <==
#!/usr/bin/python

import json
import sqlite3
import random
from . import schema_to_class as s2c
from . import dtk_in_process_qtine_fams as isofam

prob_finding_contacts = 1.0
verbose = False
symptomatic_fraction = 0.2
isofam.young_sympto_fraction = symptomatic_fraction # does this work?

def app( timestep ):
    # Do SQL query against simulation.db (?) and look for any NewlySymptomatic events this timestep
    # If there are any, for each individual, do a query for any NewInfection events within last 5 days
    # where the infector id == the id of the newly symptomatic individual.
    # Create a new campaign file which distributes SimpleVaccine to those individuals.
    # This will require creation of new method to target interventions by individual id

    # CREATE TABLE SIM_EVENTS (SIM_TIME       INT    NOT NULL,EVENT          TEXT     NOT NULL,INDIVIDUAL     INT    NOT NULL,MISC           INT    NOT NULL);

    newly_symptos = []
    victims_to_treat = []
    with sqlite3.connect('simulation_events.db') as conn:
        cur = conn.cursor()
        # test query
        query = "SELECT COUNT(*) from SIM_EVENTS;".format( timestep ) 
        cur.execute( query )
        rows = cur.fetchone()
        if rows[0] == 0:
            if verbose:
                print( "SIM_EVENTS db is empty! Ok at beginning of simulation but something might be wrong." )
            return ""

        query = "SELECT INDIVIDUAL, EVENT from SIM_EVENTS where SIM_TIME={} and ( EVENT='NewlySymptomatic' or EVENT='NewSevereCase' );".format( timestep ) 
        cur.execute( query )
        rows = cur.fetchall()
        for row in rows:
            infector = row[0]
            event    = row[1]
            if verbose:
                print( f"Found newly symptomatic individual {infector}. Look for victims..." )
            if event == "NewlySymptomatic":
                if random.random() < symptomatic_fraction:
                    newly_symptos.append( infector )
            elif event == "NewSevereCase":
                newly_symptos.append( infector )

        if len(newly_symptos)==0:
            if verbose:
                print( "Found NO newly symptomatic individuals to contact trace." )

        for infector in newly_symptos:
            if verbose:
                print( "Looking for victims of {} at time {}".format( infector, timestep ) )
            cur2 = conn.cursor()
            #query2 = "SELECT INDIVIDUAL from SIM_EVENTS where SIM_TIME<={} and SIM_TIME>{} and EVENT=='NewInfection' and MISC=={};".format( timestep, timestep-10, infector )
            query2 = "SELECT INDIVIDUAL from SIM_EVENTS where EVENT=='NewInfection' and MISC=={};".format( infector )
            cur2.execute( query2 )
            rows2 = cur2.fetchall()
            for row in rows2:
                #print( str( row ) )
                infected = row[0]
                if verbose:
                    print( f"Found infected victim {infected} of {infector}." )
                if random.random() < prob_finding_contacts:
                    victims_to_treat.append( infected )

    if len( victims_to_treat ) > 0:
        if verbose:
            print( "Going to distribute vaccines to {}.".format( victims_to_treat ) )
    else:
        if verbose:
            print( "No infectees/victims found. Not distributing any 'contact tracing' isolations this time." )
        return ""

    # DelayedIV -> MultiIV -> RASV & BroadcasttEvent
    event = s2c.get_default_campaign_event() 
    coordinator = s2c.get_class_with_defaults( "StandardEventCoordinator" )
    if coordinator is None:
        print( "s2c.get_class_with_defaults returned None. Maybe no schema.json was provided." )
        return "" 
    intervention = s2c.get_class_with_defaults( "DelayedIntervention" ) 
    act_intervention = s2c.get_class_with_defaults( "MultiInterventionDistributor" ) 
    act_act_intervention = s2c.get_class_with_defaults( "RouteAwareSimpleVaccine" ) 
    waning = s2c.get_class_with_defaults( "WaningEffectBox" )
    act_act_co_intervention = s2c.get_class_with_defaults( "BroadcastEvent" ) 

    coordinator.Target_Demographic = "ExplicitIDs"
    coordinator.Max_Cases_Per_Node  = 10
    intervention.Delay_Period_Distribution = "UNIFORM_DISTRIBUTION"
    intervention.Delay_Period_Min = 0
    intervention.Delay_Period_Max = 2
    act_act_intervention.Vaccine_Type = "Generic"
    act_act_intervention.Vaccine_Route = "TRANSMISSIONROUTE_CONTACT" 
    waning.Initial_Effect = 0.9
    waning.Box_Duration = 14
    act_act_co_intervention.Broadcast_Event = "IsolateAtHome"

    coordinator.Intervention_Config = intervention 
    event.Event_Coordinator_Config = coordinator
    intervention.Actual_IndividualIntervention_Configs = [ act_intervention ]
    act_intervention.Intervention_List = [ act_act_intervention, act_act_co_intervention ]
    act_act_intervention.Waning_Config = waning 
    
    event.Start_Day = float(timestep+1)
    event.Event_Coordinator_Config.ID_List = victims_to_treat

    campaign = {}
    campaign["Events"] = []
    campaign["Events"].append( event )

    fam_events = isofam.get_events( timestep )
    for event in fam_events:
        campaign["Events"].append( event )

    with open( "contact_trace.json", "w" ) as camp_file:
        json.dump( campaign, camp_file )
    return "contact_trace.json"
